package io.kestra.core.tasks.templating;

import io.kestra.core.models.tasks.Output;
import io.kestra.core.runners.RunContext;
import io.kestra.core.runners.RunContextFactory;
import io.kestra.core.tasks.debugs.Return;
import io.micronaut.test.extensions.junit5.annotation.MicronautTest;
import jakarta.inject.Inject;
import org.junit.jupiter.api.Test;

import java.util.Map;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.instanceOf;
import static org.hamcrest.Matchers.notNullValue;
import static org.hamcrest.core.Is.is;
import static org.junit.jupiter.api.Assertions.assertThrows;

@MicronautTest
class TemplatedTaskTest {

    @Inject
    private RunContextFactory runContextFactory;

    @Test
    void templatedType() throws Exception {
        RunContext runContext = runContextFactory.of(Map.of("type", "io.kestra.core.tasks.debugs.Return"));
        TemplatedTask templatedTask = TemplatedTask.builder()
            .id("template")
            .type(TemplatedTask.class.getName())
            .spec("""
                type: {{ type }}
                format: It's alive!""")
            .build();

        Output output = templatedTask.run(runContext);

        assertThat(output, notNullValue());
        assertThat(output, instanceOf(Return.Output.class));
        assertThat(((Return.Output)output).getValue(), is("It's alive!"));
    }

    @Test
    void templatedFlowable() {
        RunContext runContext = runContextFactory.of();
        TemplatedTask templatedTask = TemplatedTask.builder()
            .id("template")
            .type(TemplatedTask.class.getName())
            .spec("""
                type: io.kestra.core.tasks.flows.Pause
                delay: PT10S""")
            .build();

        var exception = assertThrows(IllegalArgumentException.class, () -> templatedTask.run(runContext));
        assertThat(exception.getMessage(), is("The templated task must be a runnable task"));
    }

    @Test
    void templatedTemplated() {
        RunContext runContext = runContextFactory.of();
        TemplatedTask templatedTask = TemplatedTask.builder()
            .id("template")
            .type(TemplatedTask.class.getName())
            .spec("""
                type: io.kestra.core.tasks.templating.TemplatedTask
                spec: whatever""")
            .build();

        var exception = assertThrows(IllegalArgumentException.class, () -> templatedTask.run(runContext));
        assertThat(exception.getMessage(), is("The templated task cannot be of type 'io.kestra.core.tasks.templating.TemplatedTask'"));
    }

}